import { createSlice, createAsyncThunk } from '@reduxjs/toolkit';
import api from '@/services/api';
import { setNotificationBarMessage } from './notificationBarSlice';

export const fetchProducts = createAsyncThunk('products/fetchProducts', async (_, { rejectWithValue, dispatch }) => {
    try {
        const response = await api.get('/products/');
        if (response.code !== 200) {
            throw new Error(response.message);
        }
        return response.data;
    } catch (error) {
        return rejectWithValue(error.message);
    }
});

export const addProductToCampaign = createAsyncThunk('products/addProductToCampaign', async (formData, { rejectWithValue, dispatch }) => {
    try {
        const { campaignId, productId } = formData;
        const response = await api.post(`/campaigns/${campaignId}/add_product/`, { product_id:productId });
        if (response.code !== 201 && response.code !== 200) {
            throw new Error(response.message);
        }
        dispatch(setNotificationBarMessage({ message: response.message, type: 'success' }));
        return response.data;
    } catch (error) {
        dispatch(setNotificationBarMessage({ message: error.message, type: 'error' }));
        return rejectWithValue(error.message);
    }
});

export const fetchProductDetail = createAsyncThunk('products/fetchProductDetail', async (productId, { rejectWithValue, dispatch }) => {
    try {
        const response = await api.get(`/products/${productId}/`);
        if (response.code !== 200) {
            throw new Error(response.message);
        }
        return response.data;
    } catch (error) {
        dispatch(setNotificationBarMessage({ message: error.message, type: 'error' }));
        return rejectWithValue(error.message);
    }
});

const initialState = {
    products: [],
    loading: false,
    error: null,
		productDetail: null,
};

const productSlice = createSlice({
    name: 'products',
    initialState,
    reducers: {},
    extraReducers: (builder) => {
        builder.addCase(fetchProducts.pending, (state) => {
            state.loading = true;
        });
        builder.addCase(fetchProducts.fulfilled, (state, action) => {
            state.products = action.payload;
            state.loading = false;
        })
        builder.addCase(fetchProducts.rejected, (state, action) => {
            state.error = action.payload;
            state.loading = false;
        })
        builder.addCase(addProductToCampaign.pending, (state) => {
            state.loading = true;
        })
        builder.addCase(addProductToCampaign.fulfilled, (state, action) => {
            state.products = action.payload;
            state.loading = false;
        })
        builder.addCase(addProductToCampaign.rejected, (state, action) => {
            state.error = action.payload;
            state.loading = false;
        })
        builder.addCase(fetchProductDetail.pending, (state) => {
            state.loading = true;
        })
        builder.addCase(fetchProductDetail.fulfilled, (state, action) => {
            state.productDetail = action.payload;
            state.loading = false;
        })
        builder.addCase(fetchProductDetail.rejected, (state, action) => {
            state.error = action.payload;
            state.loading = false;
        })
    },
});

export default productSlice.reducer;
